@@ -5,7 +5,7 @@ use warnings;
 
 use parent qw( Amazon::API );
 
-our $VERSION = '1.2.7';
+our $VERSION = '1.3.1';
 
 our @API_METHODS = qw{
   CreateCapacityProvider
@@ -68,7 +68,7 @@ sub new {
 
   my %options = ref( $options[0] ) ? %{ $options[0] } : @options;
 
-  $class->SUPER::new(
+  return $class->SUPER::new(
     { service     => 'ecs',
       api         => 'AmazonEC2ContainerServiceV20141113',
       api_methods => \@API_METHODS,
