@@ -1,11 +1,11 @@
-package Amazon::SSM;
+package Amazon::API::SSM;
 
 use strict;
 use warnings;
 
 use parent qw( Amazon::API );
 
-our $VERSION = '1.2.7';
+our $VERSION = '1.3.1';
 
 our @API_METHODS = qw{
   AddTagsToResource
@@ -151,7 +151,7 @@ sub new {
 
   my %options = ref( $options[0] ) ? %{ $options[0] } : @options;
 
-  my $self = $class->SUPER::new(
+  return $class->SUPER::new(
     { service      => 'ssm',
       api          => 'AmazonSSM',
       api_methods  => \@API_METHODS,
@@ -159,8 +159,6 @@ sub new {
       %options
     }
   );
-
-  return $self;
 } ## end sub new
 
 1;
