@@ -1,4 +1,4 @@
-package Amazon::STS;
+package Amazon::API::STS;
 
 use strict;
 use warnings;
@@ -16,9 +16,7 @@ our @API_METHODS = qw{
   GetSessionToken
 };
 
-our $VERSION = '1.2.7';
-
-caller or __PACKAGE__->main();
+our $VERSION = '1.3.1';
 
 sub new {
   my ( $class, @options ) = @_;
@@ -27,26 +25,14 @@ sub new {
 
   my %options = ref( $options[0] ) ? %{ $options[0] } : @options;
 
-  my $self = $class->SUPER::new(
-    { service     => 'sts',
-      version     => '2011-06-15',
-      api_methods => \@API_METHODS,
+  return $class->SUPER::new(
+    { service      => 'sts',
+      version      => '2011-06-15',
+      api_methods  => \@API_METHODS,
+      content_type => 'application/x-www-form-urlencoded',
       %options,
     }
   );
-
-  return $self;
 } ## end sub new
 
-sub main {
-  my $sts = Amazon::STS->new;
-
-  print Dumper $sts->AssumeRole(
-    [ 'RoleArn=arn:aws:iam::106518701080:role/sandbox-code-commit-role',
-      'RoleSessionName=code-commit-role'
-    ]
-  );
-
-} ## end sub main
-
 1;
