@@ -1,11 +1,11 @@
-package Amazon::SecretsManager;
+package Amazon::API::SecretsManager;
 
 use strict;
 use warnings;
 
 use parent qw( Amazon::API );
 
-our $VERSION = '1.2.7';
+our $VERSION = '1.3.1';
 
 our @API_METHODS = qw{
   CancelRotateSecret
